@@ -43,6 +43,8 @@ pub struct WsReadWrapper<T: WsStream + 'static> {
     pub binary_prefix: Option<String>,
     pub binary_base64: bool,
     pub text_base64: bool,
+    pub creation_time: ::std::time::Instant, // for measuring ping RTTs
+    pub print_rtts: bool,
 }
 
 impl<T: WsStream + 'static> AsyncRead for WsReadWrapper<T> {}
@@ -123,8 +125,23 @@ impl<T: WsStream + 'static> Read for WsReadWrapper<T> {
 
                     continue;
                 }
-                Ready(Some(OwnedMessage::Pong(_))) => {
-                    info!("Received a pong from websocket");
+                Ready(Some(OwnedMessage::Pong(buf))) => {
+                    if buf.len() == 12 {
+                        let (mut origts1, mut origts2) = ([0u8; 8], [0u8; 4]);
+                        origts1.copy_from_slice(&buf[0..8]);
+                        origts2.copy_from_slice(&buf[8..12]);
+                        let (origts1, origts2) = (u64::from_be_bytes(origts1), u32::from_be_bytes(origts2));
+                        let origts = ::std::time::Duration::new(origts1, origts2);
+                        let newts = ::std::time::Instant::now() - self.creation_time;
+                        let delta = newts.checked_sub(origts).unwrap_or_default();
+                        info!("Received a pong from websocket; RTT = {:?}", delta);
+                        if self.print_rtts {
+                            eprintln!("RTT {}.{:06} s", delta.as_secs(), delta.subsec_micros());
+                        }
+
+                    } else {
+                        warn!("Received a pong with a strange content from websocket");
+                    }
 
                     if let Some((de, intvl)) = self.pong_timeout.as_mut() {
                         de.reset(::std::time::Instant::now() + *intvl);
@@ -186,6 +203,8 @@ pub struct WsWriteWrapper<T: WsStream + 'static> {
     pub binary_prefix: Option<String>,
     pub binary_base64: bool,
     pub text_base64: bool,
+    pub close_status_code: Option<u16>,
+    pub close_reason: Option<String>,
 }
 
 impl<T: WsStream + 'static> AsyncWrite for WsWriteWrapper<T> {
@@ -193,9 +212,15 @@ impl<T: WsStream + 'static> AsyncWrite for WsWriteWrapper<T> {
         if !self.close_on_shutdown {
             return Ok(Ready(()));
         }
+        let close_data = self.close_status_code.map(|code|
+            websocket::CloseData{
+                status_code: code,
+                reason: self.close_reason.clone().unwrap_or_default()
+            }
+        );
         let mut sink = self.sink.borrow_mut();
         match sink
-            .start_send(OwnedMessage::Close(None))
+            .start_send(OwnedMessage::Close(close_data))
             .map_err(io_other_error)?
         {
             futures::AsyncSink::NotReady(_) => wouldblock(),
@@ -330,6 +355,7 @@ pub struct WsPinger<T: WsStream + 'static> {
     st: WsPingerState,
     si: MultiProducerWsSink<T>,
     t: ::tokio_timer::Interval,
+    origin: ::std::time::Instant,
     aborter: ::futures::unsync::oneshot::Receiver<()>,
 }
 
@@ -337,12 +363,14 @@ impl<T: WsStream + 'static> WsPinger<T> {
     pub fn new(
         sink: MultiProducerWsSink<T>,
         interval: ::std::time::Duration,
+        origin: ::std::time::Instant,
         aborter: ::futures::unsync::oneshot::Receiver<()>,
     ) -> Self {
         WsPinger {
             st: WsPingerState::WaitingForTimer,
             t: ::tokio_timer::Interval::new_interval(interval),
             si: sink,
+            origin,
             aborter,
         }
     }
@@ -377,7 +405,12 @@ impl<T: WsStream + 'static> ::futures::Future for WsPinger<T> {
                     }
                 },
                 StartSend => {
-                    let om = OwnedMessage::Ping(vec![]);
+                    let ts = ::std::time::Instant::now().duration_since(self.origin);
+                    let (ts1, ts2) = (ts.as_secs(), ts.subsec_nanos());
+                    let mut ts = [0; 12];
+                    ts[0..8].copy_from_slice(&ts1.to_be_bytes());
+                    ts[8..12].copy_from_slice(&ts2.to_be_bytes());
+                    let om = OwnedMessage::Ping(ts.to_vec());
                     match self.si.borrow_mut().start_send(om) {
                         Err(e) => info!("wsping: {}", e),
                         Ok(AsyncSink::NotReady(_om)) => {
@@ -420,13 +453,14 @@ pub fn finish_building_ws_peer<S>(opts: &super::Options, duplex: Duplex<S>, clos
         Mode1::Binary
     };
 
+    let now = ::std::time::Instant::now();
     let ping_aborter = if let Some(d) = opts.ws_ping_interval {
         debug!("Starting pinger");
 
         let (tx, rx) = ::futures::unsync::oneshot::channel();
 
         let intv = ::std::time::Duration::from_secs(d);
-        let pinger = super::ws_peer::WsPinger::new(mpsink.clone(), intv, rx);
+        let pinger = super::ws_peer::WsPinger::new(mpsink.clone(), intv,now, rx);
         ::tokio_current_thread::spawn(pinger);
         Some(tx)
     } else {
@@ -457,6 +491,8 @@ pub fn finish_building_ws_peer<S>(opts: &super::Options, duplex: Duplex<S>, clos
         binary_prefix: opts.ws_binary_prefix.clone(),
         binary_base64: opts.ws_binary_base64,
         text_base64: opts.ws_text_base64,
+        creation_time: now,
+        print_rtts: opts.print_ping_rtts,
     };
     let ws_sin = WsWriteWrapper{
         sink: mpsink,
@@ -467,6 +503,8 @@ pub fn finish_building_ws_peer<S>(opts: &super::Options, duplex: Duplex<S>, clos
         binary_prefix: opts.ws_binary_prefix.clone(),
         binary_base64: opts.ws_binary_base64,
         text_base64: opts.ws_text_base64,
+        close_status_code: opts.close_status_code,
+        close_reason: opts.close_reason.clone(),
     };
 
     Peer::new(ws_str, ws_sin, hup)
